/*
 *
 *
 */
#ifndef __TDM_SPAN_H__
#define __TDM_SPAN_H__

typedef enum {
	TDM_SPAN_STATE_NONE = 0,
	TDM_SPAN_STATE_INVALID	/* always last */
} tdm_span_state_t;

typedef enum {
	TDM_SPAN_EVENT_NONE = 0,
	TDM_SPAN_EVENT_SPAN_READY,	/*!< Span started */
	TDM_SPAN_EVENT_SPAN_STOPPED,	/*!< Span stopped */
	TDM_SPAN_EVENT_MAX		/*!< always last! */
} tdm_span_event_t;


/**
 * @brief Used in modules to define callbacks
 */
struct tdm_span_ops {
	tdm_status_t (* span_start)(struct tdm_span *span);	/* call module's span start handler */
	tdm_status_t (* span_stop)(struct tdm_span *span);	/* call module's span stop handler */

	tdm_status_t (* span_add_channel)(struct tdm_span *span, struct tdm_channel *chan);
	tdm_status_t (* span_remove_channel)(struct tdm_span *span, struct tdm_channel *chan);
};


struct tdm_span {
	struct tdm_object obj;

	struct tdm_span_ops *ops;

	tdm_span_state_t    state;

	unsigned int        nr_channels;
	struct dlist_head   channels;

	/* grouping */
	struct tdm_spangroup *group;
	struct dlist_head     group_span_list;
};


struct tdm_span *tdm_object_to_span(struct tdm_object *obj);
struct tdm_spangroup *tdm_span_get_spangroup(struct tdm_span *span);


struct tdm_span *tdm_span_new(const char *name);


int tdm_span_has_channel(struct tdm_span *span, struct tdm_channel *chan);
tdm_status_t tdm_span_add_channel(struct tdm_span *span, struct tdm_channel *chan);
tdm_status_t tdm_span_remove_channel(struct tdm_span *span, struct tdm_channel *chan);


tdm_span_state_t tdm_span_get_state(struct tdm_span *span);

tdm_status_t tdm_span_start(struct tdm_span *span);
tdm_status_t tdm_span_stop(struct tdm_span *span);

tdm_status_t tdm_span_queue_event(struct tdm_span *span, const tdm_span_event_t ev_type, const void *data);

#endif /* __TDM_SPAN_H__ */
